#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

from pyramid.decorator import reify
from z3c.form import field
from z3c.form.interfaces import INPUT_MODE
from zope.interface import implementer, Interface

# import packages
from pyams_form.form import ajax_config
from pyams_form.help import FormHelp
# import interfaces
from pyams_form.interfaces.form import IInnerTabForm, IFormHelp
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal import _
from pyams_portal.interfaces import IPortlet, IPortalTemplate, IPortalPage, MANAGE_TEMPLATE_PERMISSION, \
    IPortletSettings, IPortletRendererSettings, LOCAL_TEMPLATE_NAME
from pyams_portal.zmi.widget import PortletRendererFieldWidget
from pyams_skin.event import get_json_widget_refresh_event
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.url import absolute_url
from pyams_zmi.form import AdminDialogEditForm, InnerAdminEditForm
from pyams_zmi.interfaces import IPropertiesEditForm
from pyams_zmi.layer import IAdminLayer


@template_config(template='templates/portlet.pt', layer=IPyAMSLayer)
class PortletSettingsEditor(AdminDialogEditForm):
    """Portlet settings edit form"""

    prefix = 'portlet.'

    @property
    def title(self):
        translate = self.request.localizer.translate
        parent = self.configuration.__parent__
        if not IPortalTemplate.providedBy(parent):
            parent = IPortalPage(parent).template
        if parent.name == LOCAL_TEMPLATE_NAME:
            return translate(_("Local portal template - {0}")).format(translate(self.portlet.label))
        else:
            return translate(_("« {0} »  portal template - {1}")).format(parent.name,
                                                                         translate(self.portlet.label))

    legend = _("Edit portlet settings")
    dialog_class = 'modal-large'

    settings = None
    fields = field.Fields(Interface)
    edit_permission = MANAGE_TEMPLATE_PERMISSION

    @reify
    def configuration(self):
        return self.context.configuration

    @reify
    def portlet(self):
        registry = self.request.registry
        return registry.queryUtility(IPortlet, name=self.configuration.portlet_name)

    @property
    def override_label(self):
        translate = self.request.localizer.translate
        parent = self.configuration.__parent__
        if IPortalTemplate.providedBy(parent):
            return translate(_("Override template settings"))
        else:
            page = IPortalPage(parent)
            if page.inherit_parent:
                return translate(_("Override parent settings"))
            else:
                return translate(_("Override template settings"))

    def get_form_action(self):
        return absolute_url(self.configuration.__parent__, self.request, self.request.view_name)

    def get_ajax_handler(self):
        return absolute_url(self.configuration.__parent__, self.request, self.ajax_handler)


@adapter_config(context=(Interface, IAdminLayer, PortletSettingsEditor), provides=IFormHelp)
class PortletSettingsEditorFormHelpAdapter(FormHelp):
    """Portlet settings editor form help adapter"""

    def __new__(cls, context, request, view):
        configuration = view.configuration
        if (not configuration.can_inherit) or (view.mode != INPUT_MODE):
            return None
        return FormHelp.__new__(cls)

    message = _("""WARNING: Portlet properties are saved automatically when changing inherit mode!!""")
    message_format = 'rest'


@adapter_config(name='properties', context=(Interface, IPyAMSLayer, PortletSettingsEditor), provides=IInnerTabForm)
class PortletSettingsPropertiesEditor(InnerAdminEditForm):
    """Portlet settings properties editor"""

    prefix = 'portlet_properties.'

    tab_label = _("Main properties")
    legend = None

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    @property
    def fields(self):
        settings = self.parent_form.settings
        fields = field.Fields(settings).omit('__name__', 'renderer') + \
                 field.Fields(settings).select('renderer')
        fields['renderer'].widgetFactory = PortletRendererFieldWidget
        return fields

    weight = 10

    def updateWidgets(self, prefix=None):
        super(PortletSettingsPropertiesEditor, self).updateWidgets(prefix)
        if 'renderer' in self.widgets:
            self.widgets['renderer'].required = True

    def get_ajax_output(self, changes):

        def get_form_factory(context, request):
            return PortletSettingsPropertiesEditor(context, request, self.parent_form)

        output = super(PortletSettingsPropertiesEditor, self).get_ajax_output(changes)
        if 'renderer' in changes.get(IPortletSettings, ()):
            renderer = self.context.get_renderer(self.request)
            if (renderer is not None) and \
               (renderer.target_interface is None) and \
               (renderer.settings_interface is not None):
                output.update({
                    'close_form': False
                })
                output.setdefault('events', []).append(
                    get_json_widget_refresh_event(self.context, self.request, get_form_factory, 'renderer'))
                output['smallbox'] = {
                    'status': 'info',
                    'message': self.request.localizer.translate(_("You changed renderer selection. Don't omit to "
                                                                  "update new renderer properties...")),
                    'timeout': 5000
                }
        return output


#
# Portlet renderer settings edit form
#

@pagelet_config(name='renderer-properties.html', context=IPortletSettings, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
@ajax_config(name='renderer-properties.json', context=IPortletSettings, layer=IPyAMSLayer)
@implementer(IPropertiesEditForm)
class PortletRendererPropertiesEditForm(AdminDialogEditForm):
    """Portlet renderer properties edit form"""

    prefix = 'portlet_renderer_properties.'

    dialog_class = 'modal-large'
    legend = _("Edit renderer properties")
    icon_css_class = 'fa fa-fw fa-pencil-square-o'

    @property
    def fields(self):
        renderer = self.context.get_renderer(self.request)
        return field.Fields(renderer.settings_interface or Interface)

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    def getContent(self):
        return IPortletRendererSettings(self.context)
